@@ -3,37 +3,38 @@ Utility for running a prompt_toolkit application in an asyncssh server.
 """
 import asyncio
 import traceback
-from typing import Awaitable, Callable, Optional, TextIO, cast
+from typing import Any, Awaitable, Callable, Optional, TextIO, cast
 
 import asyncssh
 
 from prompt_toolkit.application.current import AppSession, create_app_session
 from prompt_toolkit.data_structures import Size
+from prompt_toolkit.eventloop import get_event_loop
 from prompt_toolkit.input import create_pipe_input
 from prompt_toolkit.output.vt100 import Vt100_Output
 
 __all__ = ["PromptToolkitSSHSession", "PromptToolkitSSHServer"]
 
 
-class PromptToolkitSSHSession(asyncssh.SSHServerSession):
+class PromptToolkitSSHSession(asyncssh.SSHServerSession):  # type: ignore
     def __init__(
         self, interact: Callable[["PromptToolkitSSHSession"], Awaitable[None]]
     ) -> None:
         self.interact = interact
         self.interact_task: Optional[asyncio.Task[None]] = None
-        self._chan = None
+        self._chan: Optional[Any] = None
         self.app_session: Optional[AppSession] = None
 
         # PipInput object, for sending input in the CLI.
         # (This is something that we can use in the prompt_toolkit event loop,
         # but still write date in manually.)
         self._input = create_pipe_input()
-        self._output = None
+        self._output: Optional[Vt100_Output] = None
 
         # Output object. Don't render to the real stdout, but write everything
         # in the SSH channel.
         class Stdout:
-            def write(s, data):
+            def write(s, data: str) -> None:
                 try:
                     if self._chan is not None:
                         self._chan.write(data.replace("\n", "\r\n"))
@@ -43,12 +44,13 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):
             def isatty(s) -> bool:
                 return True
 
-            def flush(s):
+            def flush(s) -> None:
                 pass
 
             @property
-            def encoding(s):
-                return self._chan._orig_chan.get_encoding()[0]
+            def encoding(s) -> str:
+                assert self._chan is not None
+                return str(self._chan._orig_chan.get_encoding()[0])
 
         self.stdout = cast(TextIO, Stdout())
 
@@ -62,14 +64,14 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):
             width, height, pixwidth, pixheight = self._chan.get_terminal_size()
             return Size(rows=height, columns=width)
 
-    def connection_made(self, chan):
+    def connection_made(self, chan: Any) -> None:
         self._chan = chan
 
     def shell_requested(self) -> bool:
         return True
 
     def session_started(self) -> None:
-        self.interact_task = asyncio.get_event_loop().create_task(self._interact())
+        self.interact_task = get_event_loop().create_task(self._interact())
 
     async def _interact(self) -> None:
         if self._chan is None:
@@ -98,17 +100,17 @@ class PromptToolkitSSHSession(asyncssh.SSHServerSession):
                 self._input.close()
 
     def terminal_size_changed(
-        self, width: int, height: int, pixwidth, pixheight
+        self, width: int, height: int, pixwidth: object, pixheight: object
     ) -> None:
         # Send resize event to the current application.
         if self.app_session and self.app_session.app:
             self.app_session.app._on_resize()
 
-    def data_received(self, data, datatype):
+    def data_received(self, data: str, datatype: object) -> None:
         self._input.send_text(data)
 
 
-class PromptToolkitSSHServer(asyncssh.SSHServer):
+class PromptToolkitSSHServer(asyncssh.SSHServer):  # type: ignore
     """
     Run a prompt_toolkit application over an asyncssh server.
 
