@@ -32,7 +32,7 @@ from prompt_toolkit.formatted_text.utils import (
     split_lines,
 )
 from prompt_toolkit.lexers import Lexer, SimpleLexer
-from prompt_toolkit.mouse_events import MouseEvent, MouseEventType
+from prompt_toolkit.mouse_events import MouseButton, MouseEvent, MouseEventType
 from prompt_toolkit.search import SearchState
 from prompt_toolkit.selection import SelectionType
 from prompt_toolkit.utils import get_cwidth
@@ -48,23 +48,12 @@ from .processors import (
 )
 
 if TYPE_CHECKING:
-    from prompt_toolkit.key_binding.key_bindings import KeyBindingsBase
+    from prompt_toolkit.key_binding.key_bindings import (
+        KeyBindingsBase,
+        NotImplementedOrNone,
+    )
     from prompt_toolkit.utils import Event
 
-    # The only two return values for a mouse hander are `None` and
-    # `NotImplemented`. For the type checker it's best to annotate this as
-    # `object`. (The consumer never expects a more specific instance: checking
-    # for NotImplemented can be done using `is NotImplemented`.)
-    NotImplementedOrNone = object
-    # Other non-working options are:
-    # * Optional[Literal[NotImplemented]]
-    #      --> Doesn't work, Literal can't take an Any.
-    # * None
-    #      --> Doesn't work. We can't assign the result of a function that
-    #          returns `None` to a variable.
-    # * Any
-    #      --> Works, but too broad.
-
 
 __all__ = [
     "BufferControl",
@@ -186,7 +175,7 @@ class UIContent:
         self._line_heights_cache: Dict[Hashable, int] = {}
 
     def __getitem__(self, lineno: int) -> StyleAndTextTuples:
-        " Make it iterable (iterate line by line). "
+        "Make it iterable (iterate line by line)."
         if lineno < self.line_count:
             return self.get_line(lineno)
         else:
@@ -375,9 +364,19 @@ class FormattedTextControl(UIControl):
         wrap_lines: bool,
         get_line_prefix: Optional[GetLinePrefixCallable],
     ) -> Optional[int]:
-
+        """
+        Return the preferred height for this control.
+        """
         content = self.create_content(width, None)
-        return content.line_count
+        if wrap_lines:
+            height = 0
+            for i in range(content.line_count):
+                height += content.get_height_for_line(i, width, get_line_prefix)
+                if height >= max_available_height:
+                    return max_available_height
+            return height
+        else:
+            return content.line_count
 
     def create_content(self, width: int, height: Optional[int]) -> UIContent:
         # Get fragments
@@ -455,7 +454,7 @@ class FormattedTextControl(UIControl):
                 count = 0
                 for item in fragments:
                     count += len(item[1])
-                    if count >= xpos:
+                    if count > xpos:
                         if len(item) >= 3:
                             # Handler found. Call it.
                             # (Handler can return NotImplemented, so return
@@ -537,7 +536,7 @@ class BufferControl(UIControl):
         search_buffer_control: Union[
             None, "SearchBufferControl", Callable[[], "SearchBufferControl"]
         ] = None,
-        menu_position: Optional[Callable] = None,
+        menu_position: Optional[Callable[[], Optional[int]]] = None,
         focus_on_click: FilterOrBool = False,
         key_bindings: Optional["KeyBindingsBase"] = None,
     ):
@@ -685,12 +684,12 @@ class BufferControl(UIControl):
         merged_processor = merge_processors(input_processors)
 
         def transform(lineno: int, fragments: StyleAndTextTuples) -> _ProcessedLine:
-            " Transform the fragments for a given line number. "
+            "Transform the fragments for a given line number."
             # Get cursor position at this line.
             def source_to_display(i: int) -> int:
-                """ X position from the buffer to the x position in the
+                """X position from the buffer to the x position in the
                 processed fragment list. By default, we start from the 'identity'
-                operation. """
+                operation."""
                 return i
 
             transformation = merged_processor.apply_transformation(
@@ -729,6 +728,14 @@ class BufferControl(UIControl):
         """
         buffer = self.buffer
 
+        # Trigger history loading of the buffer. We do this during the
+        # rendering of the UI here, because it needs to happen when an
+        # `Application` with its event loop is running. During the rendering of
+        # the buffer control is the earliest place we can achieve this, where
+        # we're sure the right event loop is active, and don't require user
+        # interaction (like in a key binding).
+        buffer.load_history_if_not_yet_loaded()
+
         # Get the document to be shown. If we are currently searching (the
         # search buffer has focus, and the preview_search filter is enabled),
         # then use the search document, which has possibly a different
@@ -766,11 +773,11 @@ class BufferControl(UIControl):
         self._last_get_processed_line = get_processed_line
 
         def translate_rowcol(row: int, col: int) -> Point:
-            " Return the content column for this coordinate. "
+            "Return the content column for this coordinate."
             return Point(x=get_processed_line(row).source_to_display(col), y=row)
 
         def get_line(i: int) -> StyleAndTextTuples:
-            " Return the fragments for a given line number. "
+            "Return the fragments for a given line number."
             fragments = get_processed_line(i).fragments
 
             # Add a space at the end, because that is a possible cursor
@@ -840,6 +847,18 @@ class BufferControl(UIControl):
                     buffer.exit_selection()
                     buffer.cursor_position = index
 
+                elif (
+                    mouse_event.event_type == MouseEventType.MOUSE_MOVE
+                    and mouse_event.button != MouseButton.NONE
+                ):
+                    # Click and drag to highlight a selection
+                    if (
+                        buffer.selection_state is None
+                        and abs(buffer.cursor_position - index) > 0
+                    ):
+                        buffer.start_selection(selection_type=SelectionType.CHARACTERS)
+                    buffer.cursor_position = index
+
                 elif mouse_event.event_type == MouseEventType.MOUSE_UP:
                     # When the cursor was moved to another place, select the text.
                     # (The >1 is actually a small but acceptable workaround for
@@ -847,7 +866,10 @@ class BufferControl(UIControl):
                     # the cursor can never be after the text, so the cursor
                     # will be repositioned automatically.)
                     if abs(buffer.cursor_position - index) > 1:
-                        buffer.start_selection(selection_type=SelectionType.CHARACTERS)
+                        if buffer.selection_state is None:
+                            buffer.start_selection(
+                                selection_type=SelectionType.CHARACTERS
+                            )
                         buffer.cursor_position = index
 
                     # Select word around cursor on double click.
